import edu.rice.cnc.translator.GraphDescription._
import edu.rice.cnc.api.Point

/**
 * @author Shams Imam (shams@rice.edu)
 * @since 1/2/12 5:23 PM
 */

object CholeskyCncDefinition {
  def main(args: Array[String]): Unit = {
    ApplicationName("Cholesky")
    TargetPackage("cholesky") // default package
    TargetDirectory("cholesky")

    Comment("Item Collections")
    ItemCollection[Point, Int]("numTiles")
    ItemCollection[Point, Int]("tileSize")
    ItemCollection[Point, Array[Array[Double]]]("Lkji")

    Comment("Tag Collections")
    TagCollection[Point]("singleton")
    TagCollection[Point]("controlS1")
    TagCollection[Point]("controlS2")
    TagCollection[Point]("controlS3")

    Comment("Step Prescriptions")
    Presription("singleton", List("kComputeStep"))
    Presription("controlS1", List("kjComputeStep", "s1ComputeStep"))
    Presription("controlS2", List("kjiComputeStep", "s2ComputeStep"))
    Presription("controlS3", List("s3ComputeStep"))

    Comment("Step Dependences")
    StepDependence("kComputeStep", List("numTiles"), List("controlS1"))
    StepDependence("kjComputeStep", List("numTiles"), List("controlS2"))
    StepDependence("kjiComputeStep", List(), List("controlS3"))
    StepDependence("s1ComputeStep", List("tileSize", "Lkji"), List("Lkji"))
    StepDependence("s2ComputeStep", List("tileSize", "Lkji"), List("Lkji"))
    StepDependence("s3ComputeStep", List("tileSize", "Lkji"), List("Lkji"))

    Comment("Environment Collections")
    EnvironmentDependence(
      List("tileSize", "Lkji", "numTiles", "singleton"),
      List("Lkji")
    )
  }
}