import edu.rice.cnc.translator.GraphDescription._
import edu.rice.cnc.api.Point

/**
 * @author Shams Imam (shams@rice.edu)
 * @since 1/2/12 5:23 PM
 */

object LUDecompositionCncDefinition {
  def main(args: Array[String]): Unit = {
    ApplicationName("LUDecomposition")
    TargetPackage("ludecomp") // default package
    TargetDirectory("ludecomp")

    Comment("Item Collections")
    ItemCollection[Point, Array[Array[Double]]]("tiledMatrix")
    ItemCollection[Point, Int]("tileSize")
    ItemCollection[Point, Int]("numTiles")

    Comment("Tag Collections")
    TagCollection[Point]("wholeComputationTag")
    TagCollection[Point]("pivotTileIndicesTimeTuple")
    TagCollection[Point]("rowPeripheryTileIndicesTimeTuple")
    TagCollection[Point]("columnPeripheryTileIndicesTimeTuple")
    TagCollection[Point]("subMatrixTileIndicesTimeTuple")

    Comment("Step Prescriptions")
    Presription("wholeComputationTag",
      List("createPivotTagsStep", "createColumnPeripheryTagsStep",
        "createRowPeripheryTagsStep", "createSubMatrixTagsStep"))
    Presription("columnPeripheryTileIndicesTimeTuple", List("columnPeripheryStep"))
    Presription("rowPeripheryTileIndicesTimeTuple", List("rowPeripheryStep"))
    Presription("subMatrixTileIndicesTimeTuple", List("subMatrixStep"))
    Presription("pivotTileIndicesTimeTuple", List("pivotStep"))

    Comment("Step Dependences")
    StepDependence("createPivotTagsStep",
      List("numTiles", "tileSize"),
      List("pivotTileIndicesTimeTuple"))
    StepDependence("createColumnPeripheryTagsStep",
      List("numTiles", "tileSize"),
      List("columnPeripheryTileIndicesTimeTuple"))
    StepDependence("createRowPeripheryTagsStep",
      List("numTiles", "tileSize"),
      List("rowPeripheryTileIndicesTimeTuple"))
    StepDependence("createSubMatrixTagsStep",
      List("numTiles", "tileSize"),
      List("subMatrixTileIndicesTimeTuple"))
    StepDependence("columnPeripheryStep",
      List("numTiles", "tileSize", "tiledMatrix"),
      List("tiledMatrix"))
    StepDependence("rowPeripheryStep",
      List("numTiles", "tileSize", "tiledMatrix"),
      List("tiledMatrix"))
    StepDependence("subMatrixStep",
      List("numTiles", "tileSize", "tiledMatrix"),
      List("tiledMatrix"))
    StepDependence("pivotStep",
      List("numTiles", "tileSize", "tiledMatrix"),
      List("tiledMatrix"))

    Comment("Environment Collections")
    EnvironmentDependence(
      List("numTiles", "tileSize", "tiledMatrix",
        "columnPeripheryTileIndicesTimeTuple", "rowPeripheryTileIndicesTimeTuple",
        "subMatrixTileIndicesTimeTuple", "pivotTileIndicesTimeTuple"),
      List("tiledMatrix")
    )
  }
}