package ludecomp

import edu.rice.cnc.api._
import util.continuations.cps

class UserCreateRowPeripheryTagsStepStep extends CreateRowPeripheryTagsStepStep {
  def compute(
    tag: Point,
    inNumTiles: InputCollection[Point, Int],
    inTileSize: InputCollection[Point, Int],
    outRowPeripheryTileIndicesTimeTuple: TagCollection[Point]
    ): Unit@cps[Any] = {

    val numTiles: Int = inNumTiles.get(Point.ZERO_1D)
    var time: Int = 0
    while (time < numTiles) {
      var i: Int = time + 1
      while (i < numTiles) {
        outRowPeripheryTileIndicesTimeTuple.put(Point.create(time, i, time))
        i += 1;
      }
      time += 1;
    }
  }
}

class UserCreateColumnPeripheryTagsStepStep extends CreateColumnPeripheryTagsStepStep {
  def compute(
    tag: Point,
    inNumTiles: InputCollection[Point, Int],
    inTileSize: InputCollection[Point, Int],
    outColumnPeripheryTileIndicesTimeTuple: TagCollection[Point]
    ): Unit@cps[Any] = {

    val numTiles: Int = inNumTiles.get(Point.ZERO_1D)
    var time: Int = 0
    while (time < numTiles) {
      var i: Int = time + 1
      while (i < numTiles) {
        outColumnPeripheryTileIndicesTimeTuple.put(Point.create(i, time, time))
        i += 1;
      }
      time += 1;
    }
  }
}

class UserCreatePivotTagsStepStep extends CreatePivotTagsStepStep {
  def compute(
    tag: Point,
    inNumTiles: InputCollection[Point, Int],
    inTileSize: InputCollection[Point, Int],
    outPivotTileIndicesTimeTuple: TagCollection[Point]
    ): Unit@cps[Any] = {

    val numTiles: Int = inNumTiles.get(Point.ZERO_1D)
    var i: Int = 0
    while (i < numTiles) {
      outPivotTileIndicesTimeTuple.put(Point.create(i, i, i))
      i += 1;
    }
  }
}

class UserCreateSubMatrixTagsStepStep extends CreateSubMatrixTagsStepStep {
  def compute(
    tag: Point,
    inNumTiles: InputCollection[Point, Int],
    inTileSize: InputCollection[Point, Int],
    outSubMatrixTileIndicesTimeTuple: TagCollection[Point]
    ): Unit@cps[Any] = {

    val numTiles: Int = inNumTiles.get(Point.ZERO_1D)
    var time: Int = 0
    while (time < numTiles) {
      var i: Int = time + 1
      while (i < numTiles) {
        var j: Int = time + 1
        while (j < numTiles) {
          outSubMatrixTileIndicesTimeTuple.put(Point.create(i, j, time))
          j += 1;
        }
        i += 1;
      }
      time += 1;
    }
  }
}

class UserColumnPeripheryStepStep extends ColumnPeripheryStepStep {
  def compute(
    tag: Point,
    inNumTiles: InputCollection[Point, Int],
    inTileSize: InputCollection[Point, Int],
    inTiledMatrix: InputCollection[Point, Array[Array[Double]]],
    outTiledMatrix: OutputCollection[Point, Array[Array[Double]]]
    ): Unit@cps[Any] = {

    val tileI: Int = tag(0)
    val tileJ: Int = tag(1)
    val time: Int = tag(2)
    val tileSize: Int = inTileSize.get(Point.ZERO_1D)

    val inTile = inTiledMatrix.get(Point.create(tileI, tileJ, time))
    // if column periphery
    val pivotTile = inTiledMatrix.get(Point.create(tileJ, tileJ, time + 1))
    val outTile = Array.ofDim[Double](tileSize, tileSize)

    var innerJ = 0
    while (innerJ < tileSize) {
      val currentPivot = pivotTile(innerJ)(innerJ);
      var innerI = 0
      while (innerI < tileSize) {
        var formerToBeSubtracted: Double = inTile(innerI)(innerJ)
        var k: Int = 0
        while (k < innerJ) {
          formerToBeSubtracted -= (outTile(innerI)(k) * pivotTile(k)(innerJ))
          k += 1;
        }
        outTile(innerI)(innerJ) = formerToBeSubtracted / currentPivot;
        innerI += 1
      }
      innerJ += 1
    }

    outTiledMatrix.put(Point.create(tileI, tileJ, time + 1), outTile)
  }
}

class UserPivotStepStep extends PivotStepStep {
  // ATTENTION CnC violation in-place version
  def compute(
    tag: Point,
    inNumTiles: InputCollection[Point, Int],
    inTileSize: InputCollection[Point, Int],
    inTiledMatrix: InputCollection[Point, Array[Array[Double]]],
    outTiledMatrix: OutputCollection[Point, Array[Array[Double]]]
    ): Unit@cps[Any] = {

    // iteration is the index of the pivot tile, and i ranges from 0 to numTiles-1
    val iteration: Int = tag(0)
    val time: Int = tag(2)
    val tileSize: Int = inTileSize.get(Point.ZERO_1D)
    val inTile = inTiledMatrix.get(Point.create(iteration, iteration, time))

    var i: Int = 0
    while (i < tileSize) {
      val currentPivot: Double = inTile(i)(i)
      var innerI: Int = i + 1
      while (innerI < tileSize) {
        inTile(innerI)(i) /= currentPivot
        var innerJ: Int = i + 1
        while (innerJ < tileSize) {
          inTile(innerI)(innerJ) = inTile(innerI)(innerJ) - inTile(innerI)(i) * inTile(i)(innerJ)
          innerJ += 1;
        }
        innerI += 1;
      }
      i += 1;
    }

    outTiledMatrix.put(Point.create(iteration, iteration, time + 1), inTile);
  }
}


class UserRowPeripheryStepStep extends RowPeripheryStepStep {
  def compute(
    tag: Point,
    inNumTiles: InputCollection[Point, Int],
    inTileSize: InputCollection[Point, Int],
    inTiledMatrix: InputCollection[Point, Array[Array[Double]]],
    outTiledMatrix: OutputCollection[Point, Array[Array[Double]]]
    ): Unit@cps[Any] = {

    val tileI: Int = tag(0)
    val tileJ: Int = tag(1)
    val time: Int = tag(2)
    val tileSize: Int = inTileSize.get(Point.ZERO_1D)

    val inTile = inTiledMatrix.get(Point.create(tileI, tileJ, time));
    val pivotTile = inTiledMatrix.get(Point.create(tileI, tileI, time + 1)); //if column periphery
    val outTile = Array.ofDim[Double](tileSize, tileSize)

    var innerI: Int = 0
    while (innerI < tileSize) {

      var innerJ: Int = 0
      while (innerJ < tileSize) {

        var formerToBeSubtracted: Double = inTile(innerI)(innerJ)

        var k: Int = 0
        while (k < innerI) {
          formerToBeSubtracted -= outTile(k)(innerJ) * pivotTile(innerI)(k)
          k += 1;
        }
        outTile(innerI)(innerJ) = formerToBeSubtracted

        innerJ += 1;
      }

      innerI += 1;
    }

    outTiledMatrix.put(Point.create(tileI, tileJ, time + 1), outTile)
  }
}

class UserSubMatrixStepStep extends SubMatrixStepStep {
  def compute(
    tag: Point,
    inNumTiles: InputCollection[Point, Int],
    inTileSize: InputCollection[Point, Int],
    inTiledMatrix: InputCollection[Point, Array[Array[Double]]],
    outTiledMatrix: OutputCollection[Point, Array[Array[Double]]]
    ): Unit@cps[Any] = {

    val tileI: Int = tag(0)
    val tileJ: Int = tag(1)
    val time: Int = tag(2)
    val tileSize: Int = inTileSize.get(Point.ZERO_1D)

    val inTile = inTiledMatrix.get(Point.create(tileI, tileJ, time))
    val columnTile = inTiledMatrix.get(Point.create(tileI, time, time + 1))
    val rowTile = inTiledMatrix.get(Point.create(time, tileJ, time + 1))
    val outTile = Array.ofDim[Double](tileSize, tileSize)

    var innerI: Int = 0
    while (innerI < tileSize) {
      var innerJ: Int = 0
      while (innerJ < tileSize) {

        var formerToBeSubtracted: Double = inTile(innerI)(innerJ)

        var k: Int = 0
        while (k < tileSize) {
          formerToBeSubtracted -= columnTile(innerI)(k) * rowTile(k)(innerJ)
          k += 1;
        }

        outTile(innerI)(innerJ) = formerToBeSubtracted

        innerJ += 1;
      }
      innerI += 1;
    }

    outTiledMatrix.put(Point.create(tileI, tileJ, time + 1), outTile);
  }
}

