package nqueens

import edu.rice.cnc.api._

object NQueensMain {

  private[nqueens] final val VALUES_PER_INT: Int = 6

  def main(args: Array[String]): Unit = {
    var boardSize: Int = -1

    if (args.length == 1) {
      boardSize = Integer.parseInt(args(0))
    }

    if (boardSize < 4) {
      if (args.length != 1) {
        println("Usage: NQueensMain sizeOfBoard ")
      } else {
        println("sizeOfBoard has to be at least 4")
      }
      return
    }

    try {
      val boardSizeInt: Int = boardSize
      val processPositionStep = new UserProcessPositionStepStep()
      val graph = new NQueensGraph(processPositionStep)

      graph.boardSize.put(Point.ZERO_1D, boardSize)
      val numberOfIntegersOnConfiguration: Int = (boardSize / VALUES_PER_INT) + 1
      graph.valuesPer32Bits.put(Point.ZERO_1D, numberOfIntegersOnConfiguration)

      val currentConfiguration: Array[Int] = new Array[Int](numberOfIntegersOnConfiguration + 1)
      var i: Int = 0
      while (i < numberOfIntegersOnConfiguration) {
        currentConfiguration(i) = 0
        i += 1;
      }
      //row to add to
      currentConfiguration(numberOfIntegersOnConfiguration) = 1

      val begin: Long = java.lang.System.currentTimeMillis
      graph.run(new Runnable {
        def run: Unit = {
          var currentColumn: Int = 0
          while (currentColumn < boardSizeInt) {
            currentConfiguration(0) = currentColumn
            val newPoint: Point = Point.create(currentConfiguration)
            graph.partialConfigurationTag.put(newPoint)
            currentColumn += 1;
          }
        }
      })
      val end: Long = java.lang.System.currentTimeMillis

      println("Execution time: " + (end - begin) + " milliseconds")
      println("Number of solutions: " + graph.solutions.size)
      graph.solutions.dump(System.out)

    } catch {
      case ex: Exception =>
        ex.printStackTrace()
    }


  }
}