package nqueens

import edu.rice.cnc.api._
import util.continuations.cps

object UserProcessPositionStepStep {
  private[nqueens] final val BITS_PER_VALUE: Int = 5
  private[nqueens] final val VALUES_PER_INT: Int = 6
  private[nqueens] final val BITMASK: Int = 0x1f
}

class UserProcessPositionStepStep extends ProcessPositionStepStep {
  def compute(
    tag: Point,
    inValuesPer32Bits: InputCollection[edu.rice.cnc.api.Point, Int],
    inBoardSize: InputCollection[edu.rice.cnc.api.Point, Int],
    outSolutions: OutputCollection[edu.rice.cnc.api.Point, Byte],
    outPartialConfigurationTag: TagCollection[edu.rice.cnc.api.Point]
    ): Unit@cps[Any] = {

    val size = inBoardSize.get(Point.ZERO_1D)
    val numberOfIntegersOnConfiguration = inValuesPer32Bits.get(Point.ZERO_1D)
    val configuration = Array.ofDim[Int](numberOfIntegersOnConfiguration)
    var i = 0
    while (i < numberOfIntegersOnConfiguration) {
      configuration(i) = tag(i)
      i += 1;
    }

    val rowToAdd = tag(numberOfIntegersOnConfiguration)

    var currentColumn = 0
    while (currentColumn < size) {
      if (verifyQueenPosition(configuration, rowToAdd, currentColumn)) {

        val newConfiguration = new Array[Int](numberOfIntegersOnConfiguration + 1)
        System.arraycopy(configuration, 0, newConfiguration,
          0, numberOfIntegersOnConfiguration)

        newConfiguration(numberOfIntegersOnConfiguration) = 1 + rowToAdd
        newConfiguration(rowToAdd / UserProcessPositionStepStep.VALUES_PER_INT) =
          addQueenPosition(configuration, rowToAdd, currentColumn)

        val newPoint: Point = Point.create(newConfiguration)
        if (rowToAdd + 1 != size) {
          outPartialConfigurationTag.put(newPoint)
        } else {
          outSolutions.put(newPoint, 0x3f.asInstanceOf[Byte])
        }
      }
      currentColumn += 1;
    }
  }

  private[nqueens] def addQueenPosition(
    configuration: Array[Int],
    insertToRow: Int,
    insertToColumn: Int): Int = {

    // configuration [ row / VALUES_PER_INT ] is the integer value for 6 rows which includes 'row'
    val index: Int = insertToRow / UserProcessPositionStepStep.VALUES_PER_INT
    return configuration(index) | (insertToColumn << ((insertToRow % UserProcessPositionStepStep.VALUES_PER_INT) * UserProcessPositionStepStep.BITS_PER_VALUE))
  }

  private[nqueens] def getQueenPosition(
    configuration: Array[Int],
    row: Int): Int = {

    // configuration [ row / VALUES_PER_INT ] is the integer value for 6 rows which includes 'row'
    // row % VALUES_PER_INT , is the place of 'row' in this 6 row tuple
    // ( row % VALUES_PER_INT ) * BITS_PER_VALUE , is the number of bits before this 'row' data
    var index: Int = row / UserProcessPositionStepStep.VALUES_PER_INT
    return (configuration(index) >> ((row % UserProcessPositionStepStep.VALUES_PER_INT) * UserProcessPositionStepStep.BITS_PER_VALUE)) & UserProcessPositionStepStep.BITMASK
  }

  // check the rows with smaller indexes, that already has queens and check if they attack this position
  private[nqueens] def verifyQueenPosition(
    configuration: Array[Int],
    rowToAdd: Int,
    columnToAdd: Int): Boolean = {

    var currentRow: Int = 0
    while (currentRow < rowToAdd) {
      val queenPosition: Int = getQueenPosition(configuration, currentRow)
      if (queenPosition == columnToAdd || scala.math.abs(queenPosition - columnToAdd) == (rowToAdd - currentRow)) {
        return false
      }
      currentRow += 1;
    }
    return true
  }
}